// $Id: huffman.hh,v 2.0.1.1 2004-03-16 23:28:42 pary Exp $
// Copyright (c) 1999-2002  Pavel Rychly

#ifndef FINLIB_HUFFMAN_HH
#define FINLIB_HUFFMAN_HH


#include <finlib/bitio.hh>
typedef unsigned long CodeType;

class huffman_data {
public:
    CodeType *firstcode;
    int *symbol;
    int *symidx;
    char *lens;
    CodeType *codes;
    int num, maxlen;

    enum {rw_READ = 1, rw_WRITE = 2};
    huffman_data (const char *filename, int read_write);
    huffman_data (int num, int *freqs);
    ~huffman_data ();
    bool store (char *filename);
protected:
    int generate_data (int *freqs);
    
};
    

template <class Iterator, class AtomType = unsigned char>
class read_huffman
{
protected:
    huffman_data *hd;
    read_bits<Iterator, AtomType> bits;
public:
    int operator== (const read_huffman<Iterator,AtomType> &x) const
	{
	    return bits == x.bits;
	}
    int next () {
	CodeType v = bits.get_bit();
	int l = 1;
	while (v < hd->firstcode[l]) {
	    v = 2 * v + bits.get_bit();
	    l ++;
	}
	return hd->symbol [hd->symidx[l] + v - hd->firstcode[l]];
    }
    read_huffman (huffman_data *d, Iterator memory, int skipbits=0)
	: hd (d), bits (memory, skipbits) {}
};

template <class Iterator, class AtomType = unsigned char>
class write_huffman
{
protected:
    huffman_data *hd;
    write_bits<Iterator, AtomType> bits;
public:
    void next (int x) {
	bits.binary_fix (hd->codes[x], hd->lens[x]);
    }
    int bitslen () {return bits.bitslen();}
    write_huffman (huffman_data *d, Iterator memory)
	: bits (memory), hd (d) {}
};

#endif
