// Copyright (c) 1999-2012  Pavel Rychly

#ifndef FINLIB_REVIDX_HH
#define FINLIB_REVIDX_HH

#include <finlib/config.hh>
#include <finlib/compstream.hh>
#include <finlib/binfile.hh>
#include <finlib/bititer.hh>
#include <finlib/fsop.hh>
#include <finlib/fromtof.hh>
#include <string>
#include <vector>

#if defined __GNUC__ && __GNUC__ >= 3
#include <ext/hash_map>
#else
#include <hash_map>
#endif

#if defined __GNUC__ && (__GNUC__ == 3 && __GNUC_MINOR__ >= 1 || __GNUC__ >= 4)
#define HASH_NAMESPACE __gnu_cxx::
#else
#define HASH_NAMESPACE std::
#endif

template <class RevFileClass>
class gen_cqp_revidx {
protected:
    const RevFileClass crevf;
    const RevFileClass crdxf;
    const RevFileClass ccntf;
    typedef typename RevFileClass::const_iterator IT;
    const Position maxpos;
public:
    gen_cqp_revidx (const std::string &filename, Position mpos=maxPosition)
        : crevf (filename + ".corpus.rev"), crdxf (filename + ".corpus.rdx"),
          ccntf (filename + ".corpus.cnt"), maxpos (mpos)
        {}
    //~gen_cqp_revidx ();
    FastStream *id2poss (int id) const {
        if (id >= 0) {
            int idx = crdxf [id];
            return new FIFastStream<IT> (crevf.at (idx), ccntf [id]);
        } else
            return new EmptyStream();
    }
        
    int count (int id) const {return ccntf [id];}
    int maxid () const {return ccntf.size();}
};

typedef gen_cqp_revidx<BinFile<int32_t> > cqp_revidx;
typedef gen_cqp_revidx<MapNetIntFile> orig_cqp_revidx;


class map_int_revidx {
protected:
    const MapBinFile<int32_t> crevf;
    const MapBinFile<int32_t> crdxf;
    const Position maxpos;
public:
    map_int_revidx (const std::string &filename, Position mpos=maxPosition)
        : crevf (filename + ".rev"), crdxf (filename + ".rev.idx"), 
        maxpos (mpos) {}
    FastStream *id2poss (int id) const {
        if (id >= 0) {
            int idx = crdxf [id];
            return new MemFastStream<const int> (crevf.at(idx), 
                                                 crdxf [id+1] - idx, maxpos);
        } else
            return new EmptyStream();
    }
    int count (int id) const {return crdxf [id+1] - crdxf [id];}
    int maxid () const {return crdxf.size() -1;}
};



template <class RevFileClass, class IdxFileClass=MapBinFile<uint32_t> >
class delta_revidx
{
protected:
    RevFileClass crevf;
    const IdxFileClass crdxf;
    const IdxFileClass cntf;
    const Position maxpos;
    int alignmult;
    HASH_NAMESPACE hash_map<int, NumOfPos> cnt_ovf;
public:
    delta_revidx (const std::string &filename, Position mpos=maxPosition,
                  int align=1) 
        : crevf (filename + ".rev"), crdxf (filename + ".rev.idx"), 
          cntf (filename + ".rev.cnt"), maxpos (mpos), alignmult (align) {
        if (crdxf[0] > 0) {
            // read alignmult
             DeltaPosStream<typename RevFileClass::const_iterator> 
                 dps (crevf.at (6), 1, 1024);
             alignmult = dps.next();
        }
        try {
            FromFile<int64_t> cnt64(filename + ".rev.cnt64");
            while (!cnt64)
                cnt_ovf[cnt64.get()] = cnt64.get();
        } catch (FileAccessError) {}
    }
    //~delta_revidx ();
    FastStream *id2poss (int id) const {
        if (id >= 0) {
            return new DeltaPosStream<typename RevFileClass::const_iterator> 
                (crevf.at (off_t(crdxf [id]) * off_t(alignmult)), 
                 count(id), maxpos);
        } else
            return new EmptyStream();
    }
    NumOfPos count (int id) const {
        HASH_NAMESPACE hash_map<int, NumOfPos>::const_iterator it = cnt_ovf.find(id);
        if (it != cnt_ovf.end())
            return it->second;
        return cntf [id];
    }
    int maxid () const {return cntf.size();}
    off_t data_size () const {return crevf.size();}
};


typedef delta_revidx<MapBinFile<uint8_t> > map_delta_revidx;
typedef delta_revidx<BinCachedFile<uint8_t> > file_delta_revidx;
typedef delta_revidx<BinCachedFile<uint8_t>,
                     BinFile<uint32_t> > file_file_delta_revidx;
    

#endif

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
