// Copyright (c) 2001-2012  Pavel Rychly

#ifndef FINLIB_WRITELEX_HH
#define FINLIB_WRITELEX_HH

#include <finlib/lexicon.hh>
#include <finlib/fromtof.hh>

#if defined __GNUC__ && __GNUC__ >= 3
#include <ext/hash_map>
#else
#include <hash_map>
#endif

#if defined __GNUC__ && (__GNUC__ == 3 && __GNUC_MINOR__ >= 1 || __GNUC__ >= 4)
#define HASH_NAMESPACE __gnu_cxx::
#else
#define HASH_NAMESPACE std::
#endif


class write_lexicon
{
    struct string_hash {
        HASH_NAMESPACE hash<const char*> h;
        string_hash() {}
        size_t operator()(const std::string &s) const {
            return h (s.c_str());
        }
    };
    struct int_1 {
        int i;
        int_1(): i(-1) {}
    };

    const std::string filename;
    unsigned int maxitems;
    HASH_NAMESPACE hash_map<std::string,int_1,string_hash> cache;
    HASH_NAMESPACE hash_map<std::string,int,string_hash> added;
    FILE *lexf;
    ToFile<lexpos> *lidxf;
    ToFile<int> *lsrtf;
    lexpos lexftell;
    int nextid;
    map_lexicon *lex;
    int new_item (const char *str);
    void flush_hash ();
public:
    write_lexicon (const std::string &filename, int maxitems);
    ~write_lexicon ();
    int str2id(const char *str);
    int size() {return nextid;}
};

int make_lex_srt_file (const std::string &filename);

class write_unique_lexicon
{
    const std::string filename;
    FILE *lexf;
    ToFile<lexpos> *lidxf;
    lexpos lexftell;
    int nextid;
public:
    write_unique_lexicon (const std::string &filename);
    ~write_unique_lexicon ();
    int str2id(const char *str);
    int size() {return nextid;}
};

#endif

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
